
# q-expansion of newform 1078.2.e.f, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1078
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[199, [0, -1]], [981, [1, 0]]]
aps_data = [[0, -1], [3, -3], [0, 2], [0, 0], [1, -1], [7, 0], [2, -2], [0, 0], [0, 8], [-5, 0], [4, -4], [0, -4], [-4, 0], [-8, 0], [0, 2], [6, -6], [3, -3], [0, 1], [-9, 9], [-2, 0], [4, -4], [0, -9], [-6, 0], [0, 6], [-7, 0], [-9, 9], [0, 18], [0, -2], [2, -2], [5, 0], [-19, 0], [0, 0], [-3, 3], [4, 0], [0, 10], [3, -3], [16, -16], [0, 17], [-19, 0], [0, 25], [-19, 19], [22, 0], [0, -2], [-8, 8], [15, 0], [4, -4], [-20, 0], [-4, 0], [-2, 2], [0, -28], [0, 0], [-5, 0], [0, 0], [24, 0], [0, 3], [-27, 27], [-24, 24], [0, -11], [9, -9], [-28, 0], [0, 0], [18, 0], [-2, 0], [-10, 10], [0, 1], [0, 12], [0, -13], [-12, 0], [22, -22], [-2, 0], [-18, 18], [0, 19], [4, -4], [0, -11], [1, 0], [0, 26], [-18, 18], [0, 6], [0, 9], [22, -22], [-16, 0], [-20, 0], [-25, 25], [34, 0], [0, -5], [0, -36], [-6, 0], [0, -14], [-27, 0], [-2, 0], [0, 12], [-1, 1], [40, -40], [30, 0], [0, -20], [-3, 0], [0, -22], [10, -10], [0, -10], [0, -5], [30, 0], [-26, 26], [20, -20], [0, 12], [-22, 22], [-43, 43], [-27, 0], [0, 6], [-36, 36], [-26, 0], [0, 8], [38, -38], [-15, 0], [-28, 28], [-18, 0], [27, -27], [-31, 0], [30, -30], [0, -40], [28, 0], [-22, 22], [34, 0], [0, 14], [-21, 21], [0, 33], [-27, 0], [0, 48], [0, -38], [22, 0], [0, -19], [-42, 42], [12, 0], [0, -28], [-22, 0], [0, -6], [14, 0], [24, -24], [-40, 40], [-50, 0], [-12, 12], [-8, 0], [0, 45], [-26, 26], [-4, 0], [-2, 2], [18, 0], [10, 0], [44, -44], [0, 13], [0, -4], [0, 29], [-21, 0], [-29, 0], [0, -41], [28, -28], [30, 0], [0, -36], [0, 15], [30, 0], [-11, 11], [0, 4], [14, 0], [8, 0], [0, 41], [46, -46], [-18, 18], [-10, 10], [-6, 6], [28, 0], [0, -34], [-6, 6], [16, 0], [0, -49], [38, -38], [20, -20], [59, 0], [-46, 0], [10, -10], [36, 0], [0, -50], [0, 6], [-38, 0], [16, 0], [0, 4], [26, -26], [12, -12], [-7, 7], [-9, 9], [0, -14], [-5, 5], [0, -50], [-62, 0], [0, 19], [0, -2], [44, -44], [39, -39], [-20, 20], [0, 16], [-60, 0], [0, 51], [5, -5], [53, 0], [0, 22], [44, -44], [69, 0], [12, -12], [0, 20], [0, 24], [6, 0], [65, -65], [0, -56], [-22, 0], [-40, 0], [0, 8], [-21, 21], [0, 49], [-53, 53], [42, -42], [0, -12], [-22, 0], [0, 57], [71, 0], [0, -39], [0, -16], [30, 0], [-17, 0], [0, -23], [66, -66], [0, -35], [0, -38], [52, -52], [-34, 34], [19, 0], [54, -54], [9, 0], [3, -3], [0, 6], [0, -15], [-33, 0], [-2, 0], [-36, 36], [0, -20], [48, -48], [0, -68], [0, 0], [0, 24], [-28, 0], [-17, 17], [72, -72], [69, 0], [3, 0], [0, -40], [32, -32], [26, 0], [17, -17], [0, -66], [58, -58], [64, -64], [28, 0], [0, 62], [-16, 16], [-20, 0], [30, -30], [-47, 0], [-63, 63], [0, 9], [60, 0], [11, 0], [4, 0], [10, -10], [0, 10], [-4, 4], [69, -69], [0, -32], [-34, 0], [0, -34], [0, 27], [67, -67], [0, 20], [0, 12], [39, -39], [58, -58], [3, 0], [29, 0], [0, 34], [0, 66], [-55, 55], [70, 0], [60, -60], [12, 0], [54, -54], [-13, 13], [0, 54], [34, 0], [-17, 0], [23, -23], [0, 0], [-16, 0], [0, -34], [-72, 0], [0, 14], [0, -7], [-35, 35]]
